/*
 *	app.h
 *
 *	Application class interface definition. Every OPaC application inherits its
 *	properties from this class.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_APP_H_
#define	_OPaC_APP_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (app.h);

#ifndef _OPaC_DATASLOT_H_
#include <OPaC/dataslot.h>
#endif

/****************************************************************/

/*
 *	The OPaC App class implements the application object.
 *	There is only one App object for each application.
 */

class OP_PUBLIC OPApp : public OPDataSlot
{
	
	struct TickResponder
	{
		TickResponder*	next;					//	next tick responder
		TickResponder*	prev;					//	previous tick responder
		
		OPResponderRef	responder;				//	associated responder
		Int32			remaining;				//	remaining ticks
	};
	
	static OPAppRef		current_app;			//	current application
	static Count		current_num;			//	current # applications (0 or 1)
	static Address		stack_base;				//	approximative stack base
	static Bool			is_growing_up;			//	TRUE => stack growing towards higher addresses
	
protected:
	OPListRef			window_list;			//	list of windows
	Card32				window_num;				//	application # of windows
	OPWindowRef			active_window;			//	active window (if any)
	
	OPObjectRef			builder;				//	builder object (if any)
	Bool				is_building_enabled;	//	TRUE => the user may build the interface
	
	TickResponder*		first_tick;				//	pointer to first tick responder record
	TickResponder*		last_tick;				//	pointer to last tick responder record
	
protected:
	OPApp ();
	virtual ~ OPApp ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Count TotalCount () const;					/* OVERRIDDEN */
	virtual Bool Get (Count index, OPObjectOut object);	/* OVERRIDDEN */
	virtual Bool SpecifyExtraction (OPArrayIn x);		/* OVERRIDDEN */
	
	virtual void Initialise (Bool enable_build = TRUE);
	virtual Bool SetBuilder (OPObjectIn builder);
	virtual Bool GetBuilder (OPObjectOut builder) const;
	virtual const OPObjectRef& ReturnBuilder () const;
	virtual Bool IsBuildEnabled () const;
	
	virtual void MainEventLoop ();
	virtual void HandleTick (Count ms);
	
	virtual void InsertWindow (OPWindowIn window);
	virtual void RemoveWindow (OPWindowIn window);
	
	virtual const OPListRef& ReturnWindowList () const;
	virtual Count ReturnWindowNum () const;
	
	virtual Bool GetWindow (Count index, OPWindowOut window);
	
	virtual void NotifyActiveWindow (OPWindowIn window);
	virtual const OPWindowRef& ReturnActiveWindow () const;
	
	virtual void InsertTickResponder (OPResponderIn responder, Card32 ticks);
	virtual void RemoveTickResponder (OPResponderIn responder);
	
	virtual Bool IsBuildingEnabled () const;
	
	/*
	 *	Event manager interface. Please don't call yourself, this
	 *	is intended for the application class only.
	 */
	
	static void Initialise (OPAppIn app);
	static void Kill ();
	
	static void PostQuitEvent ();
	static const OPAppRef& CurrentApp ();
	
	static Bool IsOnStack (const void* ptr);
	static Bool IsOnStack (Address ptr);
	
	OP_DEFINE_CLASS (OPApp, OPDataSlot);
};

#endif /* _OPaC_APP_H_ */
