/*
 *	archive.h
 *
 *	This file defines the archive class.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_ARCHIVE_H_
#define	_OPaC_ARCHIVE_H_

#ifndef _OPaC_OBJECT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (archive.h);

/****************************************************************/

/*
 *	The archive class is used to store and to retrieve data
 *	to/from a stream.
 */

class OP_PUBLIC OPArchive : public OPStorage
{
	
protected:
	OPArchiveRef		self;					//	reference to self
	
	OPStreamRef			output;					//	output stream
	OPStreamRef			input;					//	input stream
	
protected:
	OPArchive ();
	virtual ~ OPArchive ();
	
	virtual void StoreInstance (OPStorageIn archive);		/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn archive);		/* OVERRIDDEN */
	
public:
	virtual void Delete ();									/* OVERRIDDEN */
	
	virtual void SetOutput (OPStreamIn stream);
	virtual void SetInput (OPStreamIn stream);
	virtual void Reset ();
	
	//	The archive has to handle the row data input/output, the other data types
	//	are being handled by the super-class.
	
	virtual void StoreData (const void* data, Card32 size);	/* OVERRIDDEN */
	virtual void RestoreData (void* data, Card32 size);		/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPArchive, OPStorage);
};

/****************************************************************/

#endif /* _OPaC_ARCHIVE_H_ */
