/*
 *	key.h
 *
 *	Defines the keys.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_KEY_H_
#define	_OPaC_KEY_H_

/****************************************************************/

/*
 *	Description of the keyboard's set of shift, control,
 *	lock and meta keys. The meta keys are considered to
 *	be computer dependent and have no names associated
 *	to them.
 */

#define	OP_EV_SHIFT			(OP_BIT (0))	//	set => any shift key pressed (right or left)
#define	OP_EV_CONTROL		(OP_BIT (1))	//	set => any control key pressed (right or left)

#define	OP_EV_META_1		(OP_BIT (8))	//	set => meta key # 1 pressed
#define	OP_EV_META_2		(OP_BIT (9))	//	set => meta key # 2 pressed
#define	OP_EV_META_3		(OP_BIT (10))	//	set => meta key # 3 pressed
#define	OP_EV_META_4		(OP_BIT (11))	//	set => meta key # 4 pressed
#define	OP_EV_META_5		(OP_BIT (12))	//	set => meta key # 5 pressed
#define	OP_EV_META_6		(OP_BIT (13))	//	set => meta key # 6 pressed
#define	OP_EV_META_7		(OP_BIT (14))	//	set => meta key # 7 pressed
#define	OP_EV_META_8		(OP_BIT (15))	//	set => meta key # 8 pressed

#define	OP_EV_LEFT_SHIFT	(OP_BIT (16))	//	set => left shift key pressed
#define	OP_EV_RIGHT_SHIFT	(OP_BIT (17))	//	set => right shift key pressed
#define	OP_EV_LEFT_CONTROL	(OP_BIT (18))	//	set => left control key pressed
#define	OP_EV_RIGHT_CONTROL	(OP_BIT (19))	//	set => right control key pressed

#define	OP_EV_CAPS_LOCK		(OP_BIT (24))	//	set => caps lock key pressed
#define	OP_EV_EXTRA_LOCK_1	(OP_BIT (25))	//	set => extra lock key # 1 pressed
#define	OP_EV_EXTRA_LOCK_2	(OP_BIT (26))	//	set => extra lock key # 2 pressed
#define	OP_EV_EXTRA_LOCK_3	(OP_BIT (27))	//	set => extra lock key # 3 pressed

//	Masks for the modifier keys, the meta keys, the lock keys and the
//	geographical modifier keys are defined as follows :

#define	OP_EV_MOD_MASK		(OP_EV_SHIFT | OP_EV_CONTROL)
#define	OP_EV_META_MASK		(OP_EV_META_1 | OP_EV_META_2 | OP_EV_META_3 | OP_EV_META_4 | OP_EV_META_5 | OP_EV_META_6 | OP_EV_META_7 | OP_EV_META_8)
#define	OP_EV_LOCK_MASK		(OP_EV_CAPS_LOCK | OP_EV_EXTRA_LOCK_1 | OP_EV_EXTRA_LOCK_2 | OP_EV_EXTRA_LOCK_3)
#define	OP_EV_GEOMOD_MASK	(OP_EV_LEFT_SHIFT | OP_EV_RIGHT_SHIFT | OP_EV_LEFT_CONTROL | OP_EV_RIGHT_CONTROL)
#define	OP_EV_SHIFT_MASK	(OP_EV_SHIFT | OP_EV_LEFT_SHIFT | OP_EV_RIGHT_SHIFT)
#define	OP_EV_CONTROL_MASK	(OP_EV_CONTROL | OP_EV_LEFT_CONTROL | OP_EV_RIGHT_CONTROL)

/****************************************************************/

/*
 *	Description of the available key types. Many keyboards
 *	implement these different key types. The "value" and/or
 *	"code" fields of the event record give more details.
 */

#define	OP_KEY_NONE			((Card8)(0))	//	no key :-)
#define	OP_KEY_SPECIAL		((Card8)(1))	//	special key (PRINT SCRN, MACRO, UNDO...)
#define	OP_KEY_ALPHANUM		((Card8)(2))	//	any Unicode character
#define	OP_KEY_FUNCTION		((Card8)(3))	//	function keys F0..Fn
#define	OP_KEY_RETURN		((Card8)(4))	//	return key (that's the "<-+" key")
#define	OP_KEY_ENTER		((Card8)(5))	//	extended enter key (numeric key pad)
#define	OP_KEY_TAB			((Card8)(6))	//	tabulation key
#define	OP_KEY_BACKSPACE	((Card8)(7))	//	backspace (that's the "<--" key)
#define	OP_KEY_DELETE		((Card8)(8))	//	delete forward character
#define	OP_KEY_CURSOR		((Card8)(9))	//	cursor (up/down/left/right/page/home...)

//	The OP_KEY_CURSOR accepts many different key values, which
//	may be :

#define	OP_CURSOR_SPECIAL	((Card16)(0))	//	machine specific cursor key
#define	OP_CURSOR_UP		((Card16)(1))	//	move one line up
#define	OP_CURSOR_DOWN		((Card16)(2))	//	move one line down
#define	OP_CURSOR_LEFT		((Card16)(3))	//	move one character left
#define	OP_CURSOR_RIGHT		((Card16)(4))	//	move one character right
#define	OP_CURSOR_PAGE_UP	((Card16)(5))	//	move one page up
#define	OP_CURSOR_PAGE_DOWN	((Card16)(6))	//	move one page down
#define	OP_CURSOR_HOME		((Card16)(7))	//	move to line beginning
#define	OP_CURSOR_END		((Card16)(8))	//	move to line end
#define	OP_CURSOR_TOP		((Card16)(9))	//	move to document top
#define	OP_CURSOR_BOTTOM	((Card16)(10))	//	move to document bottom

/****************************************************************/

/*
 *	A (portable) key definition specifies the key type,
 *	value and extra modifiers.
 */

struct OPKey
{
	Card8				type;				//	key type (either alphanumeric or special)
	Card8				code;				//	special key code (ASCII upper case)
	Card16				value;				//	Unicode key value or other special value
	Card32				modifiers;			//	associated extra modifiers
};

OP_TYPECLASS (OPKey);

/****************************************************************/

#endif /* _OPaC_KEY_H_ */
