/*
 *	gridview.cxx
 *
 *	This view is a special one, since it does only draw overlays.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/grail.h>
#include <OPaC/gridview.h>

#include <math.h>

/****************************************************************/

APP_IMPLEMENT_CLASS (OPGridView);

/****************************************************************/

OPGridView::OPGridView ()
{
	OP_METHOD (OPGridView, 0);
	this->dx = 4;
	this->dy = 4;
}

OPGridView::~OPGridView ()
{
	OP_METHOD (~OPGridView, 0);
}

/****************************************************************/

static inline OPCoord
modulo (OPCoord a, OPCoord m)
{
	return (m) ? a - fmod (a, m) : a;
}

/****************************************************************/


void
OPGridView::SetGridPitch (OPCoord dx, OPCoord dy)
{
	OP_METHOD (SetGridPitch, 2);
	OP_ARG (1, OPCoord, dx);
	OP_ARG (2, OPCoord, dy);
	
	this->dx = dx;
	this->dy = dy;
}

void
OPGridView::GetGridPitch (OPCoordOut dx, OPCoordOut dy) const
{
	OP_METHOD (GetGridPitch, 2);
	OP_ARG (1, OPCoordOut, dx);
	OP_ARG (2, OPCoordOut, dy);
	
	dx = this->dx;
	dy = this->dy;
}

void
OPGridView::AlignOnGrid (OPCoordInOut x, OPCoordInOut y) const
{
	OP_METHOD (AlignOnGrid, 2);
	OP_ARG (1, OPCoordInOut, x);
	OP_ARG (2, OPCoordInOut, y);
	
	x = modulo (x, this->dx);
	y = modulo (y, this->dy);
}

/****************************************************************/

void
OPGridView::DrawOverlay (OPGrailPortIn port, OPRectIn rect)
{
	OP_METHOD (DrawOverlay, 2);
	OP_ARG (1, OPGrailPortIn, port);
	OP_ARG (2, OPRectIn, rect);
	
	OPCoord x1 = modulo (rect.X1 ()+this->dx-1, this->dx);
	OPCoord x2 = modulo (rect.X2 (), this->dx);
	OPCoord y1 = modulo (rect.Y1 ()+this->dy-1, this->dy);
	OPCoord y2 = modulo (rect.Y2 (), this->dy);
	
	OPColorPixel color = port->ReturnColorMan ()->FindColor (0.2, 0.2, 0.4);
	
	for (OPCoord y = y1; y <= y2; y += this->dy) {
		for (OPCoord x = x1; x <= x2; x += this->dx) {
			port->DrawDot (x, y, color);
		}
	}
}

void
OPGridView::Draw (OPGrailPortIn port, OPRectIn rect, OPDrawOp what)
{
	OP_METHOD (Draw, 3);
	OP_ARG (1, OPGrailPortIn, port);
	OP_ARG (2, OPRectIn, rect);
	OP_ARG (3, OPDrawOp, what);
	
	super::Draw (port, rect, what);
	
	if (what & OP_DRAW_BACKGROUND) {
		this->DrawOverlay (port, rect);
	}
}

/****************************************************************/
