/*
 *	buildnew.h
 *
 *	Support class used to create a new instance of an object.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_BUILDNEW_H_
#define _OPaC_BUILDNEW_H_

#include <OPaC/object.h>
#include <OPaC/opexport.h>

/****************************************************************/

APP_EXPORT_CLASS (OPBuildNew);

APP_DEFINE_REF (OPBuildNew, OPObject);
APP_DEFINE_ARRAY (OPBuildNew, OPObject);

/****************************************************************/

class OPBuildNew : public OPObject
{
protected:
	OPRuntimeRef		base_runtime;			//	base runtime filter
	OPWindowRef			window;					//	window containing the step-by-step wizard
	OPStepViewRef		step;					//	view used for the step-by-step wizard
	
	OPButtonRef			button_prev;			//	button pointing to the previous step
	OPButtonRef			button_next;			//	button pointing to the next/final step
	OPButtonRef			button_cancel;			//	cancel button
	
	OPRuntimeRef		user_runtime;			//	runtime selected by the user (step 1)
	OPObjectRef			user_object;			//	object created by the user (step 1)
	OPValueArray		user_values;			//	values used to initialise the object (step 2)
	OPBoxViewArray		user_initviews;			//	views used to initialise values (step 2)
	
	OPDBScrollListRef	step_1_list;			//	step 1: scroll list with runtime objects
	Count				step_2_progress;		//	step 2: progress in the sub-steps
	Count				step_2_total;			//	step 2: total # of sub-steps
	
	OPStringRef			label_finish;			//	label for the "Next" button (last step)
	Bool				hide_cancel;			//	TRUE => hide cancel when Finish is proposed
	Bool				auto_finish;			//	TRUE => automatically close on Finish
	
protected:
	OPBuildNew ();
	virtual ~ OPBuildNew ();

	virtual void SetupViewList (OPBoxViewInOut view);
	virtual void SetupViewArgs (OPBoxViewInOut view);
	virtual void SetupViewFinish (OPBoxViewInOut view);
	
	virtual void EmptyView (OPViewIn view);
	
public:
	virtual void Initialise (OPRuntimeIn base);
	virtual void DefineFinishButton (OPStringIn text, Bool auto_finish = TRUE, Bool hide_cancel = FALSE);
	
	virtual Bool GetObject (OPObjectOut object) const;
	
	virtual Bool Step1 (Bool back = FALSE);
	virtual Bool Step2 (Bool back = FALSE);
	virtual void Step2SubStep ();
	virtual Bool Step3 (Bool back = FALSE);
	virtual Bool Step4 (Bool back = FALSE);
	
	virtual void OnPrevStep ();
	virtual void OnNextStep ();
	virtual void OnCancel ();
	
	APP_DEFINE_CLASS (OPBuildNew, OPObject);
};

/****************************************************************/

#endif
