/*
 *	connectline.h
 *
 *	Draw the connecting line between two objects, possibly in
 *	two different windows.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_CONNECTLINE_H_
#define _OPaC_CONNECTLINE_H_

#include <OPaC/view.h>

/****************************************************************/

APP_EXPORT_CLASS (OPConnectLine);
APP_EXPORT_CLASS (OPConnectLineView);

/****************************************************************/

APP_DEFINE_REF (OPConnectLine, OPView);
APP_DEFINE_REF (OPConnectLineView, OPView);

/****************************************************************/

class OPConnectLine : public OPView
{
	
private:
	OPCoord				ox;
	OPCoord				oy;
	OPCoord				dx;
	OPCoord				dy;
	
	OPWindowRef			src_dot;
	OPWindowRef			src_line;
	OPWindowRef			dst_line;
	OPWindowRef			dst_rect;

	Bool				dragging;
	Bool				init_ok;
	OPCoord				drag_ox;
	OPCoord				drag_oy;
	
	OPViewRef			source_view;
	Bool				target_is_window;
	Bool				target_changed;
	OPViewRef			target_view;
	OPRect				target_box;
	OPObjectRef			target_obj;
	
protected:
	OPConnectLine ();
	virtual ~ OPConnectLine ();
	
	virtual void UpdateWindows ();
	virtual Bool FindView (OPRectOut box, OPViewOut view);
	
public:
	virtual void BeginDrag (OPViewIn source, OPCoord x, OPCoord y);
	virtual void EndDrag ();
	
	virtual void CommitConnect (OPObjectIn object);
	
	virtual void ButtonRelease (OPEventIn event);	/* OVERRIDDEN */
	virtual void DragNotify (OPEventIn event);		/* OVERRIDDEN */
	
	APP_DEFINE_CLASS (OPConnectLine, OPView);
};

/****************************************************************/

class OPConnectLineView : public OPView
{
private:
	Bool				draw_just_rect;
	
protected:
	OPConnectLineView ();
	virtual ~ OPConnectLineView ();
	
public:
	virtual void DrawJustRect (Bool yes);
	
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	APP_DEFINE_CLASS (OPConnectLineView, OPView);
};

/****************************************************************/

#endif /* _OPaC_CONNECTLINE_H_ */

